import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Qt.labs.qmlmodels

import "components"
import "view"

import Artix.Handlers
import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Форма выбора мягкого чека
 */
FullscreenDialog {
    id: root
    property DialogEventFilter eventFilter: DialogEventFilter {
        onOk: root.ok()
        onCancel: root.cancel()
    }

    SoftCheckChoiceModel { 
        id: softCheckChoiceModel
        softCheques: params.softCheques
    }

    ItemSelectionModel {
        id: selectionModel
        model: tableView.model
    }

    property list<int> columnsHorizontalAlignment: [
        Qt.AlignLeft,   // Номер заказа
        Qt.AlignLeft,   // Дата
        Qt.AlignRight,  // Сумма
    ]
    property list<int> columnsWidth: [
        tableView.width - 48 - (120 + 150),    // Номер заказа
        120,                                   // Дата
        150,                                   // Сумма
    ]

    contentItem: RowLayout {
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            ColumnLayout {
                anchors.fill: parent
                spacing: 0
                Item { Layout.preferredHeight: 16 }
                BasicLabel {
                    Style.class: "softCheckChoiceTitle"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    padding: 0; leftPadding: 24; rightPadding: 24
                    elide: Label.ElideRight
                    Tr { tr: Utils.makeTr("softCheckChoiceTitle", "Интернет-заказы") }
                }
                Item { Layout.preferredHeight: 24 }
                HorizontalHeaderView {
                    id: headerView
                    Layout.fillWidth: true
                    Layout.preferredHeight: 30
                    syncView: tableView
                    clip: true
                    resizableColumns: false
                    rowHeightProvider: function(row) { return 30 }
                    interactive: false
                    delegate: BasicLabel {
                        Style.class: "softCheckChoiceTableHeader"
                        padding: 0
                        leftPadding: column === 0 ? 24 : 0
                        rightPadding: column === tableView.columns - 1 ? 24 : 0
                        verticalAlignment: Label.AlignVCenter
                        horizontalAlignment: root.columnsHorizontalAlignment[column]
                        Tr { tr: decoration }
                        Component.onDestruction: () => { visible = false }
                    }
                }
                Line {
                    Layout.fillWidth: true
                    Layout.preferredHeight: strokeWidth
                    strokeWidth: 2
                    color: Style.parameter("color.grayDark")
                }
                TableView {
                    id: tableView
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    signal rowPressedChanged(int row, bool value)
                    model: softCheckChoiceModel
                    clip: true
                    contentWidth: width
                    columnWidthProvider: function(column) {
                        if (column === 0 || column === columns - 1)
                            return root.columnsWidth[column] + 24
                        return root.columnsWidth[column]
                    }
                    rowHeightProvider: function(row) { return 88 }
                    rowSpacing: 1
                    resizableColumns: false

                    // При масштабировании, списки могут отображаться с артефактами.
                    // Рендерим с мультисемплированием в нужном разрешении. Не знаю за счет чего, но проблему это решило.
                    layer {
                        enabled: mainWindow.multisampling
                        samples: 2
                        textureSize: Qt.size(width * mainWindow.scaleFactor, height * mainWindow.scaleFactor)
                    }

                    delegate: BasicButton {
                        id: tableDelegate
                        Style.class: "softCheckChoiceTableRowDelegate"
                        text: display
                        verticalAlignment: Qt.AlignVCenter
                        horizontalAlignment: root.columnsHorizontalAlignment[column]
                        padding: 0
                        leftPadding: column === 0 ? 24 : 0
                        rightPadding: column === tableView.columns - 1 ? 24 : 0
                        textLayout { fillWidth: true; fillHeight: true }
                        down: pressed || selectionModel.currentIndex.row === row
                        pressed: false
                        Line {
                            anchors.top: parent.bottom
                            width: tableDelegate.width
                            strokeWidth: 1
                            color: Style.parameter("color.grayBorder")
                        }
                        MouseArea {
                            id: ma
                            anchors.fill: parent
                            onPressedChanged: tableView.rowPressedChanged(row, pressed)
                            onClicked: selectionModel.setCurrentIndex(softCheckChoiceModel.index(row, column), ItemSelectionModel.SelectCurrent)
                        }
                        Connections {
                            target: tableView
                            function onRowPressedChanged(row, pressed) {
                                if (row === model.row)
                                    tableDelegate.pressed = pressed
                            }
                        }
                    }
                    ScrollBar.vertical: ScrollBar {
                        id: scrollBar
                        Timer {
                            id: scrollTimer
                            interval: 2500
                            onTriggered: parent.active = false
                        }
                        onActiveChanged: scrollTimer.start()
                    }
                }
            }
        }
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 8
                BasicButton {
                    Style.class: "softCheckChoiceCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    visible: true
                    Tr { code: "cancelButton"; defaultMessage: "Отмена" }
                    onClicked: cancel()
                }
                BasicButton {
                    Style.class: "softCheckChoiceOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    visible: selectionModel.hasSelection
                    Tr { code: "selectButton"; defaultMessage: "Выбрать" }
                    icon: Style.parameter("icon.actionPanel.arrow")
                    onClicked: ok()
                }
            }
        }
    }

    DialogHandler { id: handler }
    function ok() {
        handler.sendAnswer(selectionModel.currentIndex.row)
    }
    function cancel() {
        handler.sendAnswer(-1)
    }
}
